package com.dk3000.androidnotify

import android.content.Context
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.PluginRegistry.Registrar
import android.content.Intent
import io.flutter.plugin.common.EventChannel.EventSink
import android.widget.Toast
import android.content.pm.PackageManager
import android.content.ComponentName
import cachet.plugins.notifications.AndroidnotifyListener
import android.content.ContentValues.TAG
import android.content.IntentFilter
import android.provider.Settings.ACTION_NOTIFICATION_LISTENER_SETTINGS
import android.util.Log
import android.text.TextUtils
import android.provider.Settings;
import org.json.JSONException
import org.json.JSONObject
import android.content.BroadcastReceiver
class AndroidnotifyPlugin(context: Context) : EventChannel.StreamHandler {
  private var eventSink: EventSink? = null
  private var context: Context? = null

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = EventChannel(registrar.messenger(), "androidnotify")
      val context = registrar.activeContext()
      val plugin = AndroidnotifyPlugin(context)
      channel.setStreamHandler(plugin)
    }
  }

  //重新打开服务
  private fun toggleAndroidnotifyListenerService() {
    val pm = context?.getPackageManager()
    pm?.setComponentEnabledSetting(ComponentName(context, AndroidnotifyListener::class.java!!),
            PackageManager.COMPONENT_ENABLED_STATE_DISABLED, PackageManager.DONT_KILL_APP)
    pm?.setComponentEnabledSetting(ComponentName(context, AndroidnotifyListener::class.java!!),
            PackageManager.COMPONENT_ENABLED_STATE_ENABLED, PackageManager.DONT_KILL_APP)
    Toast.makeText(context, "重新打开服务成功", Toast.LENGTH_SHORT).show()
  }

  private fun AndroidnotifyPlugin(context: Context){
    this.context = context

    Log.d(TAG, "没有打开服务,去设置页面")
    /* Check if permission is given, if not then go to the notification settings screen. */
    if (!permissionGiven()) {
      context.startActivity(Intent(ACTION_NOTIFICATION_LISTENER_SETTINGS))
    } else {
      Log.d(TAG, "已经设置过,重新打开服务")
      toggleAndroidnotifyListenerService()
    }

    val receiver = NotificationReceiver()
    val intentFilter = IntentFilter()
    intentFilter.addAction(AndroidnotifyListener.NOTIFICATION_INTENT)
    context.registerReceiver(receiver, intentFilter)
    Log.d(TAG, "注册通知接收器")
    /* Start the notification service once permission has been given. */
    val listenerIntent = Intent(context, AndroidnotifyListener::class.java)
    context.startService(listenerIntent)
    Log.d(TAG, "开启通知监听器服务")

  }
  override fun onListen(arguments: Any?, eventSink: EventSink?) {
    this.eventSink = eventSink;
    /*
          Start the notification service once permission has been given.
         */
    val listenerIntent = Intent(context, AndroidnotifyListener::class.java)
    context?.startService(listenerIntent)
  }

  override fun onCancel(arguments: Any?) {
    this.eventSink = null;
  }
  private fun permissionGiven(): Boolean {
    val packageName = context?.getPackageName()
    val flat = Settings.Secure.getString(context?.getContentResolver(),
            "enabled_notification_listeners")
    if (!TextUtils.isEmpty(flat)) {
      val names = flat.split(":".toRegex()).dropLastWhile({ it.isEmpty() }).toTypedArray()
      for (name in names) {
        val componentName = ComponentName.unflattenFromString(name)
        val nameMatch = TextUtils.equals(packageName, componentName!!.packageName)
        if (nameMatch) {
          return true
        }
      }
    }
    return false
  }

  internal inner class NotificationReceiver : BroadcastReceiver() {
    override fun onReceive(context: Context, intent: Intent) {
      val packageName = intent.getStringExtra(AndroidnotifyListener.NOTIFICATION_PACKAGE_NAME)
      val title = intent.getStringExtra(AndroidnotifyListener.NOTIFICATION_TITLE)
      val text = intent.getStringExtra(AndroidnotifyListener.NOTIFICATION_TEXT)
      val j = JSONObject()
      try {
        j.put("packagename", packageName)
        j.put("title", title)
        j.put("text", text)
      } catch (e: JSONException) {
        e.printStackTrace()
      }
      eventSink?.success(j.toString())
    }
  }


}
